/*
 * AnyaryOperationAbstraction.hpp
 *
 *  Created on: 20. 8. 2017
 *	  Author: Jan Travnicek
 */

#include "AnyaryOperationAbstraction.hpp"

namespace abstraction {

void AnyaryOperationAbstractionImpl::attachInput ( const std::shared_ptr < abstraction::Value > & input, size_t index, bool move ) {
	if ( m_params.size ( ) < index + 1 )
		m_params.resize ( index + 1 );

	m_params [ index ] = std::make_pair ( input, move );
}

void AnyaryOperationAbstractionImpl::detachInput ( size_t index ) {
	if ( index >= m_params.size ( ) )
		throw std::invalid_argument ( "Parameter index " + ext::to_string ( index ) + " out of bounds.");

	m_params [ index ].first = nullptr;
	m_params [ index ].second = false;
}

bool AnyaryOperationAbstractionImpl::inputsAttached ( ) const {
	auto attached_lambda = [ ] ( const std::pair < std::shared_ptr < abstraction::Value >, bool > & param ) {
		return ( bool ) param.first;
	};

	return std::all_of ( m_params.begin ( ), m_params.end ( ), attached_lambda );
}

std::shared_ptr < abstraction::Value > AnyaryOperationAbstractionImpl::eval ( ) {
	if ( ! inputsAttached ( ) )
		return nullptr;

	return this->run ( );
}

} /* namespace abstraction */
