/*
 * ValueHolderInterface.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/Value.hpp>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

Value::Value ( const std::shared_ptr < abstraction::OperationAbstraction > & ref ) : m_lifeReference ( ref ) {
}

std::shared_ptr < abstraction::Value > Value::getProxyAbstraction ( ) {
	return shared_from_this ( );
}

std::string Value::getType ( ) const {
	return ext::to_string ( getTypeIndex ( ) );
}

const std::shared_ptr < abstraction::OperationAbstraction > & Value::getLifeReference ( ) const {
	return m_lifeReference;
}


std::shared_ptr < abstraction::Value > Void::cloneAsVariable ( bool, bool, bool, bool ) {
	throw std::domain_error ( "Void variables are not allowed" );
}

std::shared_ptr < abstraction::Value > Void::cloneAsValue ( bool, bool, bool, bool ) {
	return getProxyAbstraction ( );
}

ext::type_index Void::getTypeIndex ( ) const {
	return ext::type_index ( typeid ( void ) );
}

ext::set < abstraction::ParamQualifiers::ParamQualifier > Void::getTypeQualifiers ( ) const {
	return abstraction::ParamQualifiers::paramQualifiers < void > ( );
}


LazyValue::LazyValue ( const std::shared_ptr < abstraction::OperationAbstraction > & ref ) : Value ( std::move ( ref ) ) {
}

std::shared_ptr < abstraction::Value > LazyValue::cloneAsVariable ( bool, bool, bool, bool ) {
	throw std::domain_error ( "Feature not available on lazy value" );
}

std::shared_ptr < abstraction::Value > LazyValue::cloneAsValue ( bool, bool, bool, bool ) {
	throw std::domain_error ( "Feature not available on lazy value" );
}

ext::type_index LazyValue::getTypeIndex ( ) const {
	return this->getLifeReference( )->getReturnTypeIndex ( );
}

ext::set < abstraction::ParamQualifiers::ParamQualifier > LazyValue::getTypeQualifiers ( ) const {
	return this->getLifeReference ( )->getReturnTypeQualifiers ( );
}

std::shared_ptr < abstraction::Value > LazyValue::getProxyAbstraction ( ) {
	if ( cache == nullptr )
		cache = this->getLifeReference ( )->eval ( );
	return cache;
}

} /* namespace abstraction */
