/*
 * EvalHelper.cpp
 *
 *  Created on: 25. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/Registry.h>
#include <exception>
#include <common/EvalHelper.h>
#include <common/CastHelper.h>

namespace abstraction {

std::shared_ptr < abstraction::Value > EvalHelper::evalAlgorithm ( const std::string & name, const ext::vector < std::string > & templateParams, const ext::vector < std::shared_ptr < abstraction::Value > > & params, const std::vector < bool > & moves, AlgorithmCategories::AlgorithmCategory category ) {
	ext::vector < std::string > paramTypes;
	ext::vector < ext::set < abstraction::ParamQualifiers::ParamQualifier > > paramTypeQualifiers;
	for ( const std::shared_ptr < abstraction::Value > & param : params ) {
		paramTypes.push_back ( param->getType ( ) );
		paramTypeQualifiers.push_back ( param->getTypeQualifiers ( ) );
	}

	try {
		return evalAbstraction ( abstraction::Registry::getAlgorithmAbstraction ( name, templateParams, paramTypes, paramTypeQualifiers, category ), params, moves );
	} catch ( const std::exception & executionException ) {
		std::throw_with_nested ( std::runtime_error ( "Evaluation of algorithm " + name + " failed." ) );
	}
}

std::shared_ptr < abstraction::Value > EvalHelper::evalAbstraction ( std::shared_ptr < abstraction::OperationAbstraction > abstraction, const ext::vector < std::shared_ptr < abstraction::Value > > & params, const std::vector < bool > & moves ) {
	unsigned i = 0;
	ext::vector < std::shared_ptr < abstraction::Value > > casted_params;
	for ( const std::shared_ptr < abstraction::Value > & param : params ) {
		if ( abstraction::Registry::isCastNoOp ( abstraction->getParamType ( i ), param->getType ( ) ) ) {
			casted_params.push_back ( param );
		} else {
			casted_params.push_back ( abstraction::CastHelper::eval ( param, abstraction->getParamType ( i ), moves [ i ] ) );
		}
		++ i;
	}

	i = 0;
	for ( const std::shared_ptr < abstraction::Value > & param : casted_params ) {
		abstraction->attachInput ( param, i, moves [ i ] );
		++ i;
	}

	std::shared_ptr < abstraction::Value > result = abstraction->eval ( );
	if ( ! result )
		throw std::invalid_argument ( "Internal error in abstraction evaluation." );

	if ( abstraction::Registry::hasNormalize ( result->getType ( ) ) ) {
		std::shared_ptr < abstraction::OperationAbstraction > normalize = abstraction::Registry::getNormalizeAbstraction ( result->getType ( ) );
		normalize->attachInput ( result, 0, true );
		result = normalize->eval ( );
		if ( ! result )
			throw std::invalid_argument ( "Internal error in normalization." );
	}

	return result;
}

} /* namespace abstraction */
