#include <Utils.hpp>
#include <abstraction/ValueHolder.hpp>
#include <common/EvalHelper.h>

namespace Utils
{
    QRectF pointsToRect(QPointF a, QPointF b) {
        qreal minx = std::min(a.x(), b.x());
        qreal maxx = std::max(a.x(), b.x());
        qreal miny = std::min(a.y(), b.y());
        qreal maxy = std::max(a.y(), b.y());
        return {{minx, miny}, QPointF {maxx, maxy}};
    }

    std::shared_ptr<abstraction::Value> generateRandomAutomaton() {
        ext::vector<std::shared_ptr<abstraction::Value>> params;
        params.push_back(std::make_shared<abstraction::ValueHolder<size_t>>(nullptr, 5));
        params.push_back(std::make_shared<abstraction::ValueHolder<size_t>>(nullptr, 3));
        params.push_back(std::make_shared<abstraction::ValueHolder<bool>>(nullptr, true));
        params.push_back(std::make_shared<abstraction::ValueHolder<double>>(nullptr, 10.0));

        std::vector<bool> moves(params.size(), true);

        return abstraction::EvalHelper::evalAlgorithm("automaton::generate::RandomAutomatonFactory",
                                                  {},
                                                  params,
                                                  moves,
                                                  abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT);
    }

    std::shared_ptr<abstraction::Value> generateRandomGrammar() {
        ext::vector<std::shared_ptr<abstraction::Value>> params;
        params.push_back(std::make_shared<abstraction::ValueHolder<size_t>>(nullptr, 5));
        params.push_back(std::make_shared<abstraction::ValueHolder<size_t>>(nullptr, 3));
        params.push_back(std::make_shared<abstraction::ValueHolder<bool>>(nullptr, true));
        params.push_back(std::make_shared<abstraction::ValueHolder<double>>(nullptr, 30.0));

        std::vector<bool> moves(params.size(), true);

        return abstraction::EvalHelper::evalAlgorithm("grammar::generate::RandomGrammarFactory",
                                                  {},
                                                  params,
                                                  moves,
                                                  abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT);
    }
}
